-- Branched from Concreep 0.9.4
-- TODO: Add fast-inserter replacement logic too.

--For debugging
remote.add_interface("belter", {ports = function() return global.creepers end})
--To get relevant info:
-- /c game.print(serpent.line(remote.call("belter", "ports")[1]))
global.cleanup = {}

function init()
	global.creepers = {}
	global.index = 1
	local roboports = game.surfaces[1].find_entities_filtered{type="roboport"}
	for index, port in pairs(roboports) do
		addPort(port)
	end
	
	-- Create tables to rank belts, undergrounds, and splitters
	-- Tables are sorted fastest to slowest.
	global.belts = {}
	global.unders = {}
	global.splitters = {}
	global.assemblers = {"assembling-machine-3", "assembling-machine-2"} --Don't want to do a generic version of this in case of different sized assemblers.
	
	log("Belt Upgrader: Initializing")
	
	for k,v in pairs(game.entity_prototypes) do
		if v.type == "transport-belt" then
			if #global.belts == 0 then
				table.insert(global.belts, v)
				--log("Adding first belt to table.")
			else
				local added = false
				for i,n in pairs(global.belts) do
					if v.belt_speed > n.belt_speed then
						table.insert(global.belts, i, v)
						--log("Adding belt to table at position ".. i ..".")
						added = true
						break -- Oops, without this is tries to go to the next one.
					end
				end
				if not added then
					table.insert(global.belts, v)
				end
			end
		end
		if v.type == "underground-belt" then
			if #global.unders == 0 then
				table.insert(global.unders, v)
			else
				local added = false
				for i,n in pairs(global.unders) do
					if v.belt_speed > n.belt_speed then
						table.insert(global.unders, i, v)
						added = true
						break
					end
				end
				if not added then
					table.insert(global.unders, v)
				end
			end
		end
		if v.type == "splitter" then
			if #global.splitters == 0 then
				table.insert(global.splitters, v)
			else
				local added = false
				for i,n in pairs(global.splitters) do
					if v.belt_speed > n.belt_speed then
						table.insert(global.splitters, i, v)
						added = true
						break
					end
				end
				if not added then
					table.insert(global.splitters, v)
				end
			end
		end
	end
end

function checkRoboports()
	if global.creepers and #global.creepers > 0 then
		--Handle one a a time.  First manage our index.
		local creeper = global.creepers[global.index]
		
		if creeper.sleep > 0 then
			creeper.sleep = creeper.sleep + 1
			--if creeper.sleep > 480 then --This is called only up to once every 15 ticks.  Reawaken once every 2 minutes
			if creeper.sleep > 120 + creeper.cycles then --Once every 30s for debug
				creeper.sleep = 0 --This wakes it up and starts looking again.
			end
			return --End prematurely
		end
		
		if creeper then
			local roboport = creeper.roboport
			local radius = creeper.radius
			local amount = 0
			-- Place a tile per every 10 robots.
			if roboport and roboport.valid then --Check if alive!
				if roboport.logistic_network and roboport.logistic_network.valid then --Check if powered!
					if roboport.logistic_cell.construction_radius == 0 then --Not a valid creeper.
						table.remove(global.creepers, global.index)
						global.index = global.index - 1
					end
					if roboport.logistic_network.available_construction_robots > 0 then
						amount = math.max(math.floor(roboport.logistic_network.available_construction_robots / 10), 1)
						-- amount = 10
						--game.print(serpent.line(index))
						creep(global.index, amount)
					end
				end
			else -- Roboport died?
				table.remove(global.creepers, global.index)
			end
		else
			table.remove(global.creepers, global.index)
		end
		
		global.index = global.index + 1
		if global.index > #global.creepers then
			global.index = 1
		end
		--end
	end
end

function creep(index, amount)
	local creeper = global.creepers[index]
	--game.print(serpent.line(index))
	local roboport = creeper.roboport
	local triggered = false --Do we expand the radius?
	local cell = roboport.logistic_cell
	
	local radius
	local force = roboport.force
	local count = 0
	
	if cell and cell.valid then
		radius = cell.construction_radius - 0.5
	else
		return --Invalid roboport.
	end
	
	if not global.ghosts then
		global.ghosts = {}
	end
	--game.print("Upgrading up to " .. amount .. " belts.")
		-- local rando = math.random(-radius, radius) -- Pick a random point along the circumference.
		--for xx = -radius, radius, 1 do
			--for yy = -radius, radius, 1 do
				--if xx == -radius or xx == radius or yy == -radius or yy == radius then

	-- Ghosts may cause an overrun.
	-- Scary ghosts.
	local ghosts = #roboport.surface.find_entities_filtered{name = "entity-ghost", area={{roboport.position.x - radius,  roboport.position.y - radius},{roboport.position.x + radius,  roboport.position.y + radius}}}
	amount = amount - ghosts
	-- if amount <= 0 then
		-- return
	-- end
	
	local function updateCount()
		if count > 0 then
			triggered = true
		end
		amount = amount - count
		count = 0
	end
		
	--Create upgrade ghosts
	local assembler1s = roboport.surface.find_entities_filtered{name = "assembling-machine-1", area={{roboport.position.x - radius,  roboport.position.y - radius},{roboport.position.x + radius,  roboport.position.y + radius}}}
	local assembler2s = roboport.surface.find_entities_filtered{name = "assembling-machine-2", area={{roboport.position.x - radius,  roboport.position.y - radius},{roboport.position.x + radius,  roboport.position.y + radius}}}
	local assembs = {}
	for k, v in pairs(assembler1s) do
		table.insert(assembs, v)
	end
	for k, v in pairs(assembler2s) do
		table.insert(assembs, v)
	end
	local maximum = {roboport.logistic_network.get_item_count(global.assemblers[1]), roboport.logistic_network.get_item_count(global.assemblers[2])}
	
	for n,p in pairs(global.assemblers) do
		local maximum = roboport.logistic_network.get_item_count(p)
		for i, belts in pairs(assembs) do
			if maximum > 0 and count < maximum and not (belts.name == "assembling-machine-2") and not (belts.name == p) then --Replace, third check prevents replacing Assembler 3s with assembler 2s
				--game.print("Here.")
				local ghosted = belts.surface.find_entities_filtered{name = "entity-ghost", area={{belts.position.x-0.5, belts.position.y-0.5}, {belts.position.x+0.5, belts.position.y+0.5}}}
				if #ghosted == 0 then
					roboport.surface.create_entity{name="entity-ghost", position=belts.position, inner_name=p, direction = belts.direction, force=belts.force, fast_replace=true}
					--belts.order_deconstruction(force)
					count = count + 1
					--game.print(count .. " " .. amount)
					
					if count >= amount then --Out of bots.
						return
					end
				else
					count = count + 1
				end
			end
		end
	end
	
	-- for i, assembler in pairs(assemblers) do
		-- for k, v in pairs(global.assemblers) do
			-- if count < roboport.logistic_network.get_item_count(v) and count < maximum[k] then
				-- local ghosted = assembler.surface.find_entities_filtered{name = "entity-ghost", area={{assembler.position.x-0.5, assembler.position.y-0.5}, {assembler.position.x+0.5, assembler.position.y+0.5}}}
				-- if #ghosted == 0 then
					-- roboport.surface.create_entity{name="entity-ghost", position=assembler.position, inner_name=v, direction = assembler.direction, force=roboport.force, fast_replace=true}
					-- count = count + 1
					-- if count >= amount then
						-- return
					-- end
					-- break --Go to next assembler.
				-- end
			-- end
		-- end
	-- end
	
	updateCount()
	
	--Now execute fast_replace
	--Ignore the "assembler" name, this is pretty generic
	
	--local radius2 = roboport.logistic_cell.construction_radius --Check whole radius, not just the limited area.
	local function revive(ghost)

		--Previous iteration may have broken this.
		if not (ghost and ghost.valid) then
			return
		end
	
		local bots = ghost.surface.find_entities_filtered{type="construction-robot", area={{ghost.position.x-0.5, ghost.position.y-0.5}, {ghost.position.x+0.5, ghost.position.y+0.5}}}
		local assembler = roboport.surface.find_entities_filtered{type = ghost.ghost_type, area={{ghost.position.x-0.5, ghost.position.y-0.5}, {ghost.position.x+0.5, ghost.position.y+0.5}}}
		if assembler[1] and assembler[1].valid then
			--game.print("Found an assembler to replace.  Checking if bot in range.")
			for n, bot in pairs(bots) do
				local materials = ghost.ghost_prototype.items_to_place_this
				for item, ents in pairs(materials) do
					--game.print("Bot in range found, checking inventory...")
					if bot.get_item_count(item) > 0 then
						bot.remove_item({name=item})
						--Raise events
						script.raise_event(defines.events.on_robot_pre_mined, {robot=bot, entity=assembler[1]}) --For some reason assembler[1] was nil?
						
						--Just in case event destroyed something
						if not (assembler[1] and assembler[1].valid and ghost and ghost.valid) then
							log("Belt Upgrader: Interrupted by on_robot_pre_mined step.")
							return
						end
						
						--Is the ghost a belt?  If so, we'll not use fast-replace so we don't pollute the belt.
						local tmp = nil
						--Check for circuit conditions.  If so treat like a belt and insert item manually.
						if not (string.find(ghost.ghost_prototype.type, "belt") or string.find(ghost.ghost_prototype.type, "split")) and not assembler[1].get_control_behavior() then
							tmp = assembler[1].surface.create_entity{name=ghost.ghost_name, direction=assembler[1].direction, position=assembler[1].position, force=assembler[1].force, fast_replace=true}
							--Now pickup the items on the ground
							-- Can't actually pick them up now, so add to table for later.
							table.insert(global.cleanup, tmp)
						else
							--Need a special case for underground belts to preserve belt_to_ground_type
							if ghost.ghost_prototype.type == "underground-belt" then
								local item = assembler[1].prototype.mineable_properties.products[1].name
								tmp = assembler[1].surface.create_entity{name=ghost.ghost_name, direction=assembler[1].direction, position=assembler[1].position, force=assembler[1].force, fast_replace=true, spill=false, type=assembler[1].belt_to_ground_type}
								--May or may not work in 0.15.10
								bot.get_inventory(1).insert{name=item}
							else
								--This makes a lot of assumptions.  Some mods may make it crash.
								--Potential incomptability: a belt with no minable products.
								local item = assembler[1].prototype.mineable_properties.products[1].name
								tmp = assembler[1].surface.create_entity{name=ghost.ghost_name, direction=assembler[1].direction, position=assembler[1].position, force=assembler[1].force, fast_replace=true, spill=false}
								--May or may not work in 0.15.10
								bot.get_inventory(1).insert{name=item}
							end
						end
						--end
						
						if tmp then
							script.raise_event(defines.events.on_robot_built_entity, {created_entity=tmp, robot=bot})
						end
						
						-- junkpile = tmp.surface.find_entities_filtered{name="item-on-ground", area={{tmp.position.x-2, tmp.position.y-2}, {tmp.position.x+2,tmp.position.y+2}}}
						-- game.print(#junkpile)
						-- for ju, junk in pairs(junkpile) do
							-- --game.print("picking up items.")
							-- if junk.stack.name == item then
								-- junk.order_deconstruction(force)
							-- end
						-- end
						return
					end
				end
			end
		end
	end
	
	for i, ghost in pairs(global.ghosts) do
		if ghost and ghost.valid then
			if revive(ghost) then
				table.remove(global.ghosts, i)
			end
		else
			table.remove(global.ghosts, i)
		end
	end

	local insertercount = roboport.logistic_network.get_item_count("fast-inserter")
	for i, inserter in pairs(roboport.surface.find_entities_filtered{name = "inserter", area={{roboport.position.x - radius,  roboport.position.y - radius},{roboport.position.x + radius,  roboport.position.y + radius}}}) do
		if inserter.name == "inserter" and count < insertercount then
			if roboport.logistic_network.get_item_count("fast-inserter") > 0 and not inserter.to_be_deconstructed(force) then
				--local maximum = roboport.logistic_network.get_item_count("fast-inserter")

				--Done for Bobs mods
				--roboport.surface.create_entity{name="entity-ghost", position=inserter.position, inner_name="fast-inserter",  direction = inserter.direction, force=roboport.force, pickup_position=inserter.pickup_position, drop_position=inserter.drop_position}
				local ghost = roboport.surface.create_entity{name="entity-ghost", position=inserter.position, inner_name="fast-inserter",  direction = inserter.direction, force=inserter.force,
					pickup_position={inserter.pickup_position.x-inserter.position.x, inserter.pickup_position.y-inserter.position.y},
					drop_position={inserter.drop_position.x-inserter.position.x ,inserter.drop_position.y-inserter.position.y},
				}
				table.insert(global.ghosts, ghost)
				
				--Non-Bobs friendly, but works
				--roboport.surface.create_entity{name="entity-ghost", position=inserter.position, inner_name="fast-inserter", direction = inserter.direction, force=roboport.force, fast_replace=true}

				--inserter.order_deconstruction(force)
				count = count + 1
				--game.print(count .. " " .. amount)
				if count >= amount then --Out of bots.
					return
				end
			end
		end
	end
	
	updateCount()
	
	local beltlist = roboport.surface.find_entities_filtered{type = "transport-belt", area={{roboport.position.x - radius,  roboport.position.y - radius},{roboport.position.x + radius,  roboport.position.y + radius}}}
	
	--Fix for factorissimo
	if #beltlist > 0 then
		for i = #beltlist, 1, -1 do
			if string.find(beltlist[i].name, "inlet") then
				table.remove(beltlist, i)
			end
		end
	end
	
	for n,p in pairs(global.belts) do
		local maximum = roboport.logistic_network.get_item_count(p.name)
		for i, belts in pairs(beltlist) do
			if maximum > 0 and count < maximum and p.belt_speed > belts.prototype.belt_speed then --Replace
				if not belts.to_be_deconstructed(force) then
					local ghost = roboport.surface.create_entity{name="entity-ghost", position=belts.position, inner_name=p.name, direction = belts.direction, force=belts.force}
					table.insert(global.ghosts, ghost)
					--belts.order_deconstruction(force)
					count = count + 1
					--game.print(count .. " " .. amount)
					
					if count >= amount then --Out of bots.
						return
					end
				end
			end
		end
	end
				--end
			--end
		--end
		
	::endBelts::

	updateCount()
	
	beltlist = roboport.surface.find_entities_filtered{type = "underground-belt", area={{roboport.position.x - radius,  roboport.position.y - radius},{roboport.position.x + radius,  roboport.position.y + radius}}}
	--game.print("Upgrading up to " .. amount .. " underground belts.")

	for n,p in pairs(global.unders) do
		local maximum = roboport.logistic_network.get_item_count(p.name)
		for i,under in pairs(beltlist) do
			--game.print("Found an underground belt!  Looking for replacement options.")
			if maximum > 1 and count < maximum - 1 and p.belt_speed > under.prototype.belt_speed and p.max_underground_distance >= under.prototype.max_underground_distance then --Replace
				--triggered = true
				--game.print("Underground belt replacement option found.  Replacing with " .. p.name)
				if not under.to_be_deconstructed(force) then
					-- Check neighbor to see if we can replace at the same time.
					if not under.neighbours then --This is an orphaned underground belt
						local ghost = roboport.surface.create_entity{name="entity-ghost", position=under.position, inner_name=p.name, direction = under.direction, type=under.belt_to_ground_type, force=under.force}
						table.insert(global.ghosts, ghost)
						--under.order_deconstruction(force)
						count = count + 1
						break
					else
					-- We're assuming if it's in logistic range, it's in construction range.
					-- This will cause some weird edge cases mods (super long underground belts or roboports with the same construction radius as logistic networks)
					--if #under.neighbours == 1 and under.surface.find_logistic_network_by_position(under.position, force) then
					--if #under.neighbours == 1 then
					
						local ghost = roboport.surface.create_entity{name="entity-ghost", position=under.position, inner_name=p.name, direction = under.direction, type=under.belt_to_ground_type, force=under.force}
						table.insert(global.ghosts, ghost)
						-- under.order_deconstruction(force)
						
						--Do Neighbor
						local ghost = roboport.surface.create_entity{name="entity-ghost", position=under.neighbours.position, inner_name=p.name, direction = under.neighbours.direction, type=under.neighbours.belt_to_ground_type, force=under.force}
						table.insert(global.ghosts, ghost)
						-- under.neighbours[1].order_deconstruction(force)
						
						count = count + 2
						--game.print(count .. " " .. amount)

						if count >= amount - 1 then --Out of bots.
							return
						end
					end
				end
			end
		end
	end
	
	updateCount()
	
	beltlist = roboport.surface.find_entities_filtered{type = "splitter", area={{roboport.position.x - radius,  roboport.position.y - radius},{roboport.position.x + radius,  roboport.position.y + radius}}}
	--game.print("Upgrading up to " .. amount .. " splitters.")
		
	for n,p in pairs(global.splitters) do
		for i, belts in pairs(beltlist) do
			local maximum = roboport.logistic_network.get_item_count(p.name)
			if maximum > 0 and count < maximum and p.belt_speed > belts.prototype.belt_speed then --Replace
				--triggered = true
				if not belts.to_be_deconstructed(force) then
					local ghost = roboport.surface.create_entity{name="entity-ghost", position=belts.position, inner_name=p.name, direction = belts.direction, force=belts.force}
					table.insert(global.ghosts, ghost)
					-- belts.order_deconstruction(force)
					count = count + 1
					--game.print(count .. " " .. amount)

					if count >= amount then --Out of bots.
						return
					end
				end
			end
		end
	end
	
	-- Loop is over and we're still here?  Increase the raidus.
	--if count == 0 then
	if not triggered then
		creeper.radius = creeper.radius + 1
		local cell = roboport.logistic_cell
		if cell and cell.valid and creeper.radius > cell.construction_radius then
			-- Stop growing.
			creeper.sleep = 1 --Can't sleep if we need to rescan for ghosts.
			creeper.radius = 1
			creeper.cycles = creeper.cycles + 1
		end
	end
		--end
	--end
end

function junkCleanup()
	if not global.cleanup then
		global.cleanup = {}
		return
	end
	for i=#global.cleanup, 1, -1 do
		local tmp = global.cleanup[i] 
		junkpile = tmp.surface.find_entities_filtered{name="item-on-ground", area={{tmp.position.x-1.5, tmp.position.y-1.5}, {tmp.position.x+1.5,tmp.position.y+1.5}}}
		--game.print(#junkpile)
		for ju, junk in pairs(junkpile) do
			--game.print("picking up items.")
			--if junk.stack.name == item then
			junk.order_deconstruction(tmp.force)
			--end
		end
		table.remove(global.cleanup, i)
	end
end

function roboports(event)
	if global.creepers then
		if event.created_entity and event.created_entity.valid and event.created_entity.type == "roboport" then
			addPort(event.created_entity)
		end
	else
		init()
	end
end

function addPort(robo)
	local surface = robo.surface
	table.insert(global.creepers, {roboport = robo, radius = 1, sleep = 0, cycles=0})
end	

script.on_event(defines.events.on_built_entity, function(event)
	roboports(event)
end)

script.on_event(defines.events.on_robot_built_entity, function(event)
	roboports(event)
end)

script.on_event(defines.events.on_tick, function(event)
	if (event.tick+45) % 120 == 0 then
		junkCleanup()
	end
	if (event.tick+43) % 120 == 0 then
		checkRoboports()
	end
end)

script.on_configuration_changed(function()
	init()
end)