
script.on_init(function()

	-- return if this game is not new-game
	if (game.tick > 10) then return end

	local rng = game.create_random_generator()
	local tiles = {}
	local surface = game.surfaces[1]
	local map_settings = surface.map_gen_settings

	-- check map size (detect campaign)
	if (map_settings.width < 256 or map_settings.height < 256) then return end


	-- starting island size
	local startingAreaList = 
	{
		none = 48,
		["very-low"] = 48,
		low = 96,
		normal = 128,
		high = 172,
		["very-high"] = 256,
	}
	local startingArea = startingAreaList[map_settings.starting_area] or 128
	startingArea = math.floor(startingArea * 1.4)

	local map = {{},{}}
	for i = 1, startingArea*2+16 do
		map[1][i] = {}
		map[2][i] = {}
		for j = 1, startingArea*2+16 do
			map[1][i][j] = 0
			map[2][i][j] = 0
		end
	end


-- ---------------------------
	-- build starting island

--[[
	local diff
	local c_size = math.floor(startingArea/8)
	for i=1,64 do
		local pos_x = math.floor((rng()^2)*startingArea - startingArea/2)
		local pos_y = math.floor((rng()^2)*startingArea - startingArea/2)

		for x = -c_size, c_size do
			for y = -c_size, c_size do
				diff = (x^2 + y^2)/(c_size^2)
				if (diff < rng()) then
					map[1][x+startingArea+8+pos_x][y+startingArea+8+pos_y] = 1
				end
			end
		end
	end
]]--

	-- island shape
	local i_max = {}
	for i= 1, 8 do
		i_max[i] = math.floor(rng()*startingArea/2 + startingArea/2)
		if (i%2 == 0) then
			i_max[i] = math.floor(i_max[i]/1.4)
		end
	end


	-- fill octagon
	local mos = 80*(startingArea/180)
	for x = 0, i_max[2] do
		for y = 0, startingArea do
			diff = ((i_max[2]-i_max[1])/i_max[2])*x+i_max[1] -y
			if (x <= y and diff > rng()*mos) then
				map[1][x+startingArea+8][y+startingArea+8] = 1
			end
		end
	end

	for y = 0, i_max[2] do
		for x = 0, startingArea do
			diff = ((i_max[2]-i_max[3])/i_max[2])*y+i_max[3] -x
			if (y <= x and diff > rng()*mos) then
				map[1][x+startingArea+8][y+startingArea+8] = 1
			end
		end
	end
--
	for x = 0, i_max[4] do
		for y = 0, startingArea do
			diff = ((i_max[4]-i_max[5])/i_max[4])*x+i_max[5] -y
			if (x <= y and diff > rng()*mos) then
				map[1][x+startingArea+8][-y+startingArea+8] = 1
			end
		end
	end

	for y = 0, i_max[4] do
		for x = 0, startingArea do
			diff = ((i_max[4]-i_max[3])/i_max[4])*y+i_max[3] -x
			if (y <= x and diff > rng()*mos) then
				map[1][x+startingArea+8][-y+startingArea+8] = 1
			end
		end
	end
--
	for x = 0, i_max[6] do
		for y = 0, startingArea do
			diff = ((i_max[6]-i_max[5])/i_max[6])*x+i_max[5] -y
			if (x <= y and diff > rng()*mos) then
				map[1][-x+startingArea+8][-y+startingArea+8] = 1
			end
		end
	end

	for y = 0, i_max[6] do
		for x = 0, startingArea do
			diff = ((i_max[6]-i_max[7])/i_max[6])*y+i_max[7] -x
			if (y <= x and diff > rng()*mos) then
					map[1][-x+startingArea+8][-y+startingArea+8] = 1
			end
		end
	end
--
	for x = 0, i_max[8] do
		for y = 0, startingArea do
			diff = ((i_max[8]-i_max[1])/i_max[8])*x+i_max[1] -y
			if (x <= y and diff > rng()*mos) then
					map[1][-x+startingArea+8][y+startingArea+8] = 1
			end
		end
	end

	for y = 0, i_max[8] do
		for x = 0, startingArea do
			diff = ((i_max[8]-i_max[7])/i_max[8])*y+i_max[7] -x
			if (y <= x and diff > rng()*mos) then
					map[1][-x+startingArea+8][y+startingArea+8] = 1
			end
		end
	end


-- ---------------------------
	-- smoothing (very slow)
	local td = { 0, 10, 20, 20, 50, 80, 80, 90, 100}
	local score, pre, nex
	for i=0,19 do
		pre = i%2 +1
		nex = (i+1)%2 +1
		for x=2, startingArea*2+15 do
			for y=2, startingArea*2+15 do
				score = map[pre][x-1][y-1] + map[pre][x-1][y] + map[pre][x-1][y+1] + 
						map[pre][x  ][y-1] +                    map[pre][x  ][y+1] + 
						map[pre][x+1][y-1] + map[pre][x+1][y] + map[pre][x+1][y+1] + 1

				if (td[score] > rng()*100) then
					map[nex][x][y] = 1
				else
					map[nex][x][y] = 0
				end
			end
		end
	end

	for x=2, startingArea*2+15 do
		for y=2, startingArea*2+15 do
			score = map[1][x-1][y-1] + map[1][x-1][y] + map[1][x-1][y+1] + 
					map[1][x  ][y-1] + map[1][x  ][y] + map[1][x  ][y+1] + 
					map[1][x+1][y-1] + map[1][x+1][y] + map[1][x+1][y+1]

			if (score > 3) then 
				map[2][x][y] = 1
			else
				map[2][x][y] = 0
			end
		end
	end


-- ---------------------------
	-- draw starting island
	for x = 1,startingArea*2+16 do
		for y = 1, startingArea*2+16 do
			if (map[2][x][y] == 1 and surface.get_tile( x-startingArea-8 , y-startingArea-8 ).valid) then
				local tilename = surface.get_tile( x-startingArea-8 , y-startingArea-8 ).name
				if ( tilename == "deepwater" or tilename == "water" or tilename == "deepwater-green" or tilename == "water-green") then
					table.insert( tiles , { name = "grass" , position = { x = x-startingArea-8 , y = y-startingArea-8 }})
				end
			end
		end
	end
	surface.set_tiles(tiles)


-- ---------------------------
	-- regenerate ore-patches etc.

	for name, _ in pairs(map_settings.autoplace_controls) do
		if name ~= "enemy-base" then
			pcall(surface.regenerate_entity, name)
		end
	end

	local tree_name={
		"tree-01","tree-02","tree-02-red","tree-03","tree-04","tree-05",
		"tree-06","tree-06-brown","tree-07","tree-08","tree-08-brown","tree-08-red",
		"tree-09","tree-09-brown","tree-09-red",
		"dead-tree","dry-tree","dead-grey-trunk","dry-hairy-tree","dead-dry-hairy-tree"
	}
	for _, name in pairs(tree_name) do
--		surface.regenerate_entity(name)
		pcall(surface.regenerate_entity, name)
	end

	local decorative_name = {
		"brown-asterisk","green-asterisk","red-asterisk",
		"green-coral-mini","brown-coral-mini","orange-coral-mini",
--		"grass-medium","grass-dry","dirt","dirt-dark",
--		"sand","sand-dark","red-desert","red-desert-dark",
	}

	for _, name in pairs(decorative_name) do
--		surface.regenerate_decorative(name)
		pcall(surface.regenerate_decorative, name)
	end

end)



