local autoplace_utils = require("autoplace_utils")
local offset_str = {
	["Very-High"] = 4990,
	["High"] = 4800,
	["Medium"] = 4000,
	["Low"] = 3000,
	["Very-Low"] =2000
}

local offset = offset_str[settings.startup["kap-islands-water-level"].value] or 4000
--local offset = 4000


local function water_autoplace_settings_override(from_depth, rectangles)
	local ret =
	{
		{
			-- Water and deep water have absolute priority. We simulate this by giving
			-- them absurdly large influence
			influence = 1e3 + from_depth,
			elevation_optimal = -5000 - from_depth + offset,
			elevation_range = 5000,
			elevation_max_range = 5000, -- everywhere below elevation 0 and nowhere else
		}
	}

	if rectangles == nil then
	ret[2] = { influence = 1 }
	end

	autoplace_utils.peaks(rectangles, ret)

	return { peaks = ret }
end


data.raw["tile"]["deepwater"].autoplace = water_autoplace_settings_override(250+offset/16)
data.raw["tile"]["deepwater-green"].autoplace = water_autoplace_settings_override(250+offset/16, {{{35, 1}, {15, 0.7}}})
data.raw["tile"]["water"].autoplace = water_autoplace_settings_override(0)
data.raw["tile"]["water-green"].autoplace = water_autoplace_settings_override(0, {{{35, 1}, {15, 0.7}}})

data.raw["map-gen-presets"]["default"]["kap_islands"] =
{
	order = "w",
	basic_settings =
	{
		autoplace_controls = {
			["uranium-ore"] = {
				size = "high"
			},
			["iron-ore"] = {
				frequency = "high",
				size = "high"
			},
			["copper-ore"] = {
				frequency = "high",
				size = "high"
			},
			["coal"] = {
				frequency = "high",
				size = "high"
			},
			["stone"] = {
				frequency = "high",
				size = "high"
			},
			["crude-oil"] = {
				frequency = "high",
				size = "high"
			},
			["uranium-ore"] = {
				size = "high"
			},
		},
		terrain_segmentation = "very-high",
		water = "very-high",
	}
}



